/*
CartoTypeResultCode.h
Copyright 2011-2021 CartoType Ltd.
See www.cartotype.com for more information.
*/

#ifndef CartoTypeResultCode_h__
#define CartoTypeResultCode_h__

/**
\file
CartoType result codes and the result type.
*/

/**
CartoType result codes.
CartoType functions actually return integers typed as CTResult, which
may be compared with these values.
*/ 
typedef enum CartoTypeResultCode
    {
    /** No error, a successful result. */
    CTErrorNone = 0,
    
    /**
    CTErrorGeneral is used where an error has occurred but no other CartoType
    error is applicable. For example, this error code can be used
    when FreeType returns an error code for illegal TrueType hinting instructions.
    */
    CTErrorGeneral = 1,
    
    /** Use CTErrorNoMemory when an attempted memory allocation fails. */
    CTErrorNoMemory = 2,
    
    /**
    CTErrorEndOfData is returned by iterators and streams when no more data
    is available. It may be treated as an error or not depending on the
    circumstances.
    */
    CTErrorEndOfData = 3,
    
    /**
    CTErrorTextUnmodifiable is returned when an attempt is made to
    modify an unmodifiable string.
    */
    CTErrorTextUnmodifiable = 4,
    
    /**
    CTErrorTextTruncated is returned when text inserted into
    a string is too long for a fixed-size
    buffer.
    */
    CTErrorTextTruncated = 5,
    
    /**
    CTErrorNotFound is returned whenever a resource like a file
    is not found, or a search fails. 
    It may be treated as an error or not depending on the
    circumstances.
    */
    CTErrorNotFound = 6,
    
    /**
    The code CTErrorUnimplemented is used for functions that have
    not yet been implemented, or for circumstances or options
    within functions that remain unimplemented because they
    are deemed to be either rarely or never useful.
    */
    CTErrorUnimplemented = 7,
    
    /**
    If the graphics system detects an invalid outline, such as one
    that starts with a cubic control point, it returns
    CTErrorInvalidOutline.
    */
    CTErrorInvalidOutline = 8,
    
    /**
    The code CTErrorIo should be used for unexpected read or write
    errors on files or other data streams. This is for where you
    attempt to read data that should be there, but the OS says
    it's unavailable (e.g., someone has taken the CD out of the drive
    or the network is down). 
    */
    CTErrorIo = 9,
    
    /**
    CTErrorCorrupt is returned when illegal values are found when reading data
    from a file or other serialized form.
    */
    CTErrorCorrupt = 10,
    
    /**
    This error is returned by database accessors, typeface accessors, etc.,
    to indicate that this data format is unknown to a particular accessor.
    It is not really an error but tells the caller to try the next
    accessor in the list.
    */
    CTErrorUnknownDataFormat = 11,
    
    /**
    This error is returned by transformation inversion if the transformation 
    has no inverse.
    */
    CTErrorNoInverse = 12,
    
    /**
    This error is returned by the projection system if the projection 
    causes overflow.
    */
    CTErrorProjectionOverflow = 13,
    
    /**
    The code CTErrorCancel is returned to cancel an asynchronous
    operation that uses callbacks. The callback or virtual function
    returns this code. the caller must terminate further processing
    on receipt of any error, but this one indicates that there has been
    no actual error, but the handler has canceled the operation.
    */
    CTErrorCancel = 14,
    
    /**
    This error is returned when an invalid argument has been given to a function.
    */
    CTErrorInvalidArgument = 15,
    
    /**
    This error is returned by a data reader when it can not deal with the data version.
    */
    CTErrorUnknownVersion = 16,
    
    /**
    This error is returned by the base library when reading data or calculations result
    in overflow.
     */
    CTErrorOverflow = 17,
    
    // ABOLISHED: CTErrorOverlap = 18,
    
    /**
    The error code returned by line intersection algorithms when the lines are parallel.
    */
    CTErrorParallelLines = 19,
    
    // ABOLISHED: CTErrorPathLengthExceeded = 20,
    
    // ABOLISHED: CTErrorMaxTurnExceeded = 21,
    
    /**
    An attempt was made to draw a bitmap needing a color palette
    but no palette was provided.
    */
    CTErrorNoPalette = 22,
    
    /**
    An attempt was made to insert a duplicate object into a collection
    class that does not allow duplicates.
    */
    CTErrorDuplicate = 23,
    
    /**
    The projection for converting latitude and longitude to map coordinates
    is unknown or unavailable.
    */
    CTErrorNoProjection = 24,
    
    /**
    A palette is full and no new entries can be added.
    */
    CTErrorPaletteFull = 25,
    
    /**
    The dash array for drawing strokes is invalid.
    */
    CTErrorInvalidDashArray = 26,
    
    /**
    The central-path system failed to create a usable path.
    */
    CTErrorCentralPath = 27,
    
    /**
    A route was needed by the navigation system
    but no route was available.
    */
    CTErrorNoRoute = 28,
    
    /**
    There was an attempt to zoom beyond the legal zoom limits for a map.
    */
    CTErrorZoomLimitReached = 29,
    
    /**
    There was an attempt to project a map object that had already been projected.
    */
    CTErrorAlreadyProjected = 30,
    
    /**
    Conditions are too deeply nested in the style sheet.
    */
    CTErrorConditionsTooDeeplyNested = 31,
    
    /**
    There was an attempt to use a null font for drawing text.
    */
    CTErrorNullFont = 32,
    
    /**
    An attempt to read data from the internet failed.
    */
    CTErrorInternetIo = 33,
    
    /**
    Division by zero in an interpreted expression.
    */
    CTErrorDivideByZero = 34,
    
    /**
    A transform failed because an argument or result was out of range.
    */
    CTErrorTransformFailed = 35,
    
    /**
    Reading a bitmap from PNG format failed.
    */
    CTErrorPngRead = 36,
    
    /**
    Reading a bitmap from JPG format failed.
    */
    CTErrorJpgRead = 37,
    
    /**
    An object did not intersect a specified region.
    */
    CTErrorNoIntersection = 38,
    
    /**
    An operation was interrupted, for example by another thread writing to a shared flag.
    */
    CTErrorInterrupt = 39,
    
    /**
    There was an attempt to use map databases of incompatible point format,
    datum or axis orientations to draw a map or find objects in a map.
    */
    CTErrorMapDataBaseFormatMismatch = 40,
    
    /** A key supplied for encryption was too short. */
    CTErrorEncryptionKeyTooShort = 41,
    
    /** No encryption key has been set. */
    CTErrorNoEncryptionKey = 42,
    
    /** A code for standard emergency messages. */
    CTErrorEmergencyMessage = 43,
    
    /** A code for standard alert messages. */
    CTErrorAlertMessage = 44,
    
    /** A code for standard critical messages. */
    CTErrorCriticalMessage = 45,
    
    /** A code for standard error messages. */
    CTErrorErrorMessage = 46,
    
    /** A code for standard warning messages. */
    CTErrorWarningMessage = 47,
    
    /** A code for standard notice messages. */
    CTErrorNoticeMessage = 48,
    
    /** A code for standard information messages. */
    CTErrorInfoMessage = 49,
    
    /** A code for standard debug messages. */
    CTErrorDebugMessage = 50,
    
    /** A function has been called which is available only when navigating. */
    CTErrorNotNavigating = 51,
    
    /** The global framework object does not exist. */
    CTErrorNoFramework = 52,
    
    /** The global framework object already exists. */
    CTErrorFrameworkAlreadyExists = 53,
    
    /** A string was not transliterable. */
    CTErrorUntransliterable = 54,
    
    /** Writing a bitmap to PNG format failed. */
    CTErrorPngWrite = 55,
    
    /** There was an attempt to write to a read-only map database. */
    CTErrorReadOnlyMapDataBase = 56,
    
    /** There was an error in the PROJ.4 projection library error other than a projection overflow. */
    CTErrorProj4 = 57,

    /** No route could be created because there were no roads near the start point of a route section. */
    CTErrorNoRoadsNearStartOfRoute = 59,

    /** No route could be created because there were no roads near the end point of a route section. */
    CTErrorNoRoadsNearEndOfRoute = 60,

    /** No route could be created because the start and end point were not connected. */
    CTErrorNoRouteConnectivity = 61,
    
    /** An unsupported feature was requested from the XML parser. */
    CTErrorXmlFeatureNotSupported = 62,
    
    /** A map file was not found. */
    CTErrorMapNotFound = 63,

    /** A font file was not found. */
    CTErrorFontNotFound = 64,

    /** A style sheet was not found. */
    CTErrorStyleSheetNotFound = 65,
    
    /** An attempt was made to set a route attribute on a feature info object that is not a route. */
    CTErrorFeatureInfoIsNotRoute = 66,

    /** An attempt was made to set a non-route attribute on a feature info object that is a route. */
    CTErrorFeatureInfoIsRoute = 67,

    /** No route data is available. */
    CTErrorNoRouteData = 68,

    /** Too many fonts are used in a style sheet. */
    CTErrorTooManyFontsInStyleSheet = 69,

    /** Too many box styles are used in a style sheet. */
    CTErrorTooManyBoxStylesInStyleSheet = 70,

    /** The number of standard error codes. */
    CTStandardErrorCodeCount = 71,
    
    /**
    The start of the range of errors in XML parsing or in the style sheet format, such as a syntactically incorrect dimension or color.
    The top byte is 0x10 for style sheet errors, or in the range 0x11 to 0x2A for Expat XML parser error codes.

    The low three bytes give the error location: one byte for the column number (clamped to 0...255) two bytes for the line number (clamped to 0...65535).
    */
    CTErrorXmlRangeStart = 0x10000000,

    /** The end of the range of errors in XML parsing or in the style sheet format. */
    CTErrorXmlRangeEnd = 0x2AFFFFFF,
    
    /**
    The base of error codes for returning SQLite errors.
    The SQLite error code is placed in the lower two bytes.
    */
    CTErrorSQLite = 0x30000000

    } CartoTypeResultCode;

/** A result code typed as an integer. */
typedef uint32_t CTResult;

#endif

